/*
 * YLineParser.cpp
 * Project: part of belofte - A Promising Chess Program
 *
 * Created by Yves De Billoëz on 14/10/2006.
 */

#include "belofte.h"

#include "YLineParser.h"

/**
 * Parse a line and return result into resultbuffer, the resultbuffer is a two-dimensional array.
 * Make sure to call the cleaning function after all data has been handled.
 * failing so will cause a memory leak !!!
 * A result buffer will contain maximum iMaxElements that each can be maximum iMaxLen in size.
 * iMaxLen should include a zero terminating character.
 * @see YParse_Line_Clean
 * @see strtok_r
 */
int YParse_Line(const char *szLine, const char *szSeparator,
        char *aResultParts[],
        const int iMaxElements, const size_t iMaxLen) {
  char *szTmp; // working array
  char *szResult; // working pointer, used for mallocs
  char *szNextIndex;
  int iNumTokens = 0;

  // create a temporary working buffer
  szTmp = (char *) malloc(strlen(szLine) + 1);
  strcpy(szTmp, szLine);

  // seek for first occurence
  szResult = strtok_r(szTmp, szSeparator, &szNextIndex);
  if (szResult != NULL) {
    // fill in buffer
    aResultParts[iNumTokens] = (char *) malloc(MAX(strlen(szResult), iMaxLen));
    strncpy(aResultParts[iNumTokens], szResult, iMaxLen);
    aResultParts[iNumTokens][iMaxLen - 1] = '\0';

    // at least one token is present, so start to search for the next one
    for (iNumTokens = 1; iNumTokens < iMaxElements; iNumTokens++) {
      szResult = strtok_r(NULL, szSeparator, &szNextIndex);
      if (szResult == NULL) {
        // no more elements
        aResultParts[iNumTokens] = szResult;
        break;
      }
      // fill in buffer
      aResultParts[iNumTokens] = (char *)
              malloc(MAX(strlen(szResult), iMaxLen));
      strncpy(aResultParts[iNumTokens], szResult, iMaxLen);
      aResultParts[iNumTokens][iMaxLen - 1] = '\0';
    }
  }

  free(szTmp);

  return iNumTokens;
}

/**
 * clean memory allocated by YParse_Line
 */
void YParse_Line_Clean(char *aResultParts[], const int iNumTokens) {
  int i = 0;
  while (i < iNumTokens) {
    free(aResultParts[i++]);
  }
}

#if defined(__WIN32__)

/*
 * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Högskolan
 * (Royal Institute of Technology, Stockholm, Sweden).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the Institute nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

char *strtok_r(char *s1, const char *s2, char **lasts) {
  char *ret;

  if (s1 == NULL)
    s1 = *lasts;
  while (*s1 && strchr(s2, *s1))
    ++s1;
  if (*s1 == '\0')
    return NULL;
  ret = s1;
  while (*s1 && !strchr(s2, *s1))
    ++s1;
  if (*s1)
    *s1++ = '\0';
  *lasts = s1;
  return ret;
}

#endif