/*
 * File:   YThread.cpp
 * Project: part of belofte - A Promising Chess Program
 * Author: yves
 *
 * Created on 26 juin 2017, 13:20
 */

#include "belofte.h"

#include "YThread.h"

/** create new thread and start running it
 */
YThread_t YThread_Create(YThreadFun_t threadfun(YThreadFunParam_t),
        const YThreadParam_t param, YThreadRc_t * rcode) {
  YThread_t tid;
  YThreadRc_t rc;

  rc = pthread_create((pthread_t *) & tid, 0, threadfun, param);
  if (rcode)
    *rcode = rc;

  return tid;
}

/** cancel thread from inside thread
 */
void YThread_Terminate(YThread_t * tid) {
  YThreadRc_t rcode;

  *tid = 0;

  pthread_exit(&rcode);
}

/** cancel thread from outside thread
 * result is thrown away except if thread was stopped before
 */
YThreadRc_t YThread_Destroy(YThread_t tid, UNUSED YThreadRc_t * rcode) {
  int rc = pthread_cancel(*tid);
  return rc;
}

/** sleep for a number of milleseconds, can be called inside any thread
 * and even be used without any threads, will be aborted by SIG
 */
void YThread_Snooze(const long milliSeconds) {
  struct timespec wait;
  wait.tv_sec = milliSeconds / (1000 * 1000);
  wait.tv_nsec = (milliSeconds % (1000 * 1000)) * 1000;
  nanosleep(&wait, NULL);
}

// eof
