/** YUtils.cpp
 * Project: part of belofte - A Promising Chess Program
 * Utility functions
 */

#include "belofte.h"

#include "YUtils.h"

int countspaces(char *str) {
  int spaces;
  char *s = str;

  for (spaces = 0; *s; s++) {
    if (*s != ' ') continue;
    spaces++;
  }

  return spaces;
}

/**
 * Find if string is present in array of strings
 * @param aArray
 * @param size of array
 * @param szToFind
 * @return index + 1, or zero if not found
 */
int findinstringarray(char *aArray[], const int iNumElements,
        const char *szToFind) {
  for (int i = 0; i < iNumElements; i++) {
    if (strcmp(aArray[i], szToFind) == 0) return i + 1;
  }
  return 0;
}

/**
 * add character at end of string
 * @param szDest string pointer where character is to be added
 * @param cChar character to add
 */
void add_char(char *szDest, const char cChar) {
  int iLen = (int) strlen(szDest);

  szDest[iLen] = cChar;
  szDest[iLen + 1] = '\0';
}

/**
 * insert a single character at position iPos
 */
void ins_char(char *szDest, const char cChar, const int iPos) {
  char szBuffer[MAX_DATANAME_LENGTH];
  int i;

  for (i = 0; i < iPos; i++)
    szBuffer[i] = szDest[i];
  szBuffer[iPos] = cChar;
  szBuffer[iPos + 1] = '\0';
  strncat(szBuffer, szDest + i, MAX_DATANAME_LENGTH);

  strncpy(szDest, szBuffer, MAX_DATANAME_LENGTH);
}

void stat_logadd(const char *szLine, const int nDir) {
  FILE *fpLog;

  struct tm *p_tm;
  time_t l_tm;
  char szTime[80];

  l_tm = time(NULL); // get system time
  p_tm = localtime(&l_tm); // return local time in the form of tm structure
  strftime(szTime, sizeof (szTime), "%Y.%m.%d %T", p_tm);

  fpLog = fopen(LOGNAME, "a+");
  if (fpLog != NULL) {
    if (nDir == 1) {
      fprintf(fpLog, "%s --> %s\n", szTime, szLine);
    } else if (nDir == 2) {
      fprintf(fpLog, "%s   > %s\n", szTime, szLine);
    } else if (nDir == 3) {
      fprintf(fpLog, "%s ... %s\n", szTime, szLine);
    } else {
      fprintf(fpLog, "%s <-- %s\n", szTime, szLine);
    }
    fclose(fpLog);
  }
}

/**
 * read line
 * @return -1 if there is an end of file or error
 */
int file_readline(char * szBuffer, FILE *pFile, const size_t nSize) {
  unsigned int iIndex = 0;
  int iCurrent;
  unsigned char cCurrent;
  int nCommentLevel = 0;
  int nReadChars = 0;

  if (nSize < 2) return 0;
  szBuffer[iIndex] = 0;

  while (!feof(pFile)) {
    iCurrent = fgetc(pFile);
    if (iCurrent == EOF) break;

    cCurrent = (unsigned char) iCurrent;
    nReadChars++;
    if (cCurrent == '\n') {
      if (nCommentLevel == 0) {
        break;
      }
    } else if (cCurrent == '\r') {
      // we skip this character
      // for Mac OS 9 and prior files this will fail
      // on Mac OX X and DOS/UNIX, no problem
    } else if (cCurrent == '{') {
      nCommentLevel++;
    } else if (cCurrent == '}') {
      nCommentLevel--;
    } else if ((nCommentLevel > 0) && (cCurrent == ']')) {
      // break out of error reading
      nCommentLevel = 0;
    } else {
      if (nCommentLevel == 0) {
        szBuffer[iIndex++] = cCurrent;
        szBuffer[iIndex] = 0;
      }
    }
    if (iIndex + 1 >= nSize) break;
    if (nReadChars > 5000) {
      stat_logadd("Reading more than 5000 chars", 0);
      return -1; // make sure to break out if comment malformed
    }
  }
  if (feof(pFile)) return -1;

  return 0;
}

/**
 * read one character at a time
 */
int file_readchar(FILE *pFile) {
  unsigned char aBuf[2];

  // break on end of file
  if (feof(pFile)) return -1;

  if (read(fileno(pFile), aBuf, 1) < 1) return -1;

  return (int) aBuf[0];
}

// eof