/***********************************************************************************
 * File: alg1_moves.cpp
 * Project: part of belofte - A Promising Chess Program
 * Author: yves
 ***********************************************************************************/

#include "belofte.h"

#include "board.h"

#include "alg1_moves.h"
#include "alg1_eval.h"
#include "alg1_board.h"

// --------------------------------------------------------------------

extern t_alg1_board m1_board;
extern t_alg1_moves m1_movelist;
extern int m1_aPieceValues[FIELDVALUE_SIZE];
extern t_score m1_centervalues[FIELDVALUE_SIZE];

extern game_t game;

// --------------------------------------------------------------------

static int compare_score(const void *m1, const void *m2);

/**
 * Give initial value to moves and sort them according to estimated
 * position in bestmove list
 * @param iMoves
 */
void moves_initorder(const int iMoves) {

  int i;
  t_score fValue;

  posid cSourceField;
  posid cDestField;

  // TODO: add killer statistics
  for (i = 0; i < iMoves; i++) {

    fValue = -SCORE_ALMOST_WON + 188;

    cSourceField = toupper(m1_board.m_fields[m1_movelist.m1_moves[i].ucFrom]);
    cDestField = toupper(m1_board.m_fields[m1_movelist.m1_moves[i].ucTo]);

    if (m1_movelist.m1_moves[i].bNonSilent) fValue += 10.0;
    if (m1_movelist.m1_moves[i].bFlag) fValue += 9.0;
    if ((cSourceField == PIECE_QUEEN) && (game.curply < OPENING_LENGTH)) fValue -= 5.0;

    // penalty for high value piece taking some other piece
    fValue += (t_score) (2.0 * (abs(m1_aPieceValues[(int) cDestField]))
            - ((t_score) abs(m1_aPieceValues[(int) cSourceField]) / 10.0));

    fValue += m1_centervalues[m1_movelist.m1_moves[i].ucTo]
            - m1_centervalues[m1_movelist.m1_moves[i].ucFrom];

    m1_movelist.m1_moves[i].bValue = fValue;
  }

  moves_resort();
}

/**
 * Give initial value to moves and sort them according to estimated
 * position in bestmove list, place non-silent moves on top of list
 * @param iMoves
 * @return number of non-silent moves
 */
int moves_qsinitorder(const int iMoves) {
  int i;
  int iNonSilent = 0;
  t_score fValue;

  posid cSourceField;
  posid cDestField;

  // TODO: add killer statistics
  for (i = 0; i < iMoves; i++) {

    fValue = -SCORE_ALMOST_WON + 188;

    cSourceField = toupper(m1_board.m_fields[m1_movelist.m1_moves[i].ucFrom]);
    cDestField = toupper(m1_board.m_fields[m1_movelist.m1_moves[i].ucTo]);

    if (m1_movelist.m1_moves[i].bNonSilent) {
      fValue += 50.0;
      iNonSilent++;
    }
    if (m1_movelist.m1_moves[i].bFlag) fValue += 9.0;
    if ((cSourceField == PIECE_QUEEN) && (game.curply < OPENING_LENGTH)) fValue -= 5.0;

    // penalty for high value piece taking some other piece
    fValue += (t_score) (2.0 * (abs(m1_aPieceValues[(int) cDestField]))
            - ((t_score) abs(m1_aPieceValues[(int) cSourceField]) / 10.0));

    fValue += m1_centervalues[m1_movelist.m1_moves[i].ucTo]
            - m1_centervalues[m1_movelist.m1_moves[i].ucFrom];

    m1_movelist.m1_moves[i].bValue = fValue;
  }

  moves_resort();
  return iNonSilent;
}

/**
 * In between 2 new depth calculations, clear scores so if
 * alfa cut-off comes, no invalid scores are left
 */
void moves_clearscores(void) {
  int i;

  for (i = 0; i < m1_movelist.m1_count; i++) {
    m1_movelist.m1_moves[i].bValue = -SCORE_ALMOST_WON;
  }
}

/**
 * Rearange move sorting based on actual value
 */
void moves_resort(void) {
  qsort(&(m1_movelist.m1_moves[0]), m1_movelist.m1_count, sizeof (t_alg1_move), compare_score);
}

// --------------------------------------------------------------------

static int compare_score(const void *m1, const void *m2) {
  if (((const t_alg1_move *) m1)->bValue > ((const t_alg1_move *) m2)->bValue) return -1;
  if (((const t_alg1_move *) m1)->bValue < ((const t_alg1_move *) m2)->bValue) return 1;
  return 0;
}

// --------------------------------------------------------------------
// eof
