/*
 * File:   alg2_board.cpp
 * Project: part of belofte - A Promising Chess Program
 * Author: yves
 *
 * Created on 6 mars 2019, 00:19
 */

#include "alg2_board.h"

extern board_t m_board;

/**
 * board class
 * the board is always represented as white, inverting it when required
 * for move generation and modifying move when returning
 */

alg2_board::alg2_board(const alg2_board& orig) {
  moves = orig.moves;
}

alg2_board::~alg2_board() {

}

/**
 * access public m_board which contains ui representation of board
 */
void alg2_board::copyboardfromui() {
  for (rank_t iRank = 0; iRank < 8; iRank++) {
    for (col_t iCol = 0; iCol < 8; iCol++) {
      switch (m_board.m_fields[iCol][iRank]) {
        case FIELD_EMPTY:
          boardPiece[iRank][iCol] = new alg2_pieceEmpty();
          break;
        case PIECE_PAWN:
          boardPiece[iRank][iCol] = new alg2_piecePawn(m_board.m_fields[iCol][iRank]);
          break;
        case BLACK_PAWN:
          boardPiece[iRank][iCol] = new alg2_piecePawnBlack(m_board.m_fields[iCol][iRank]);
          break;
        case PIECE_ROOK:
          boardPiece[iRank][iCol] = new alg2_pieceRook(m_board.m_fields[iCol][iRank]);
          break;
        case BLACK_ROOK:
          boardPiece[iRank][iCol] = new alg2_pieceRookBlack(m_board.m_fields[iCol][iRank]);
          break;
        case PIECE_KNIGHT:
          boardPiece[iRank][iCol] = new alg2_pieceKnight(m_board.m_fields[iCol][iRank]);
          break;
        case BLACK_KNIGHT:
          boardPiece[iRank][iCol] = new alg2_pieceKnightBlack(m_board.m_fields[iCol][iRank]);
          break;
        case PIECE_BISHOP:
          boardPiece[iRank][iCol] = new alg2_pieceBishop(m_board.m_fields[iCol][iRank]);
          break;
        case BLACK_BISHOP:
          boardPiece[iRank][iCol] = new alg2_pieceBishopBlack(m_board.m_fields[iCol][iRank]);
          break;
        case PIECE_QUEEN:
          boardPiece[iRank][iCol] = new alg2_pieceQueen(m_board.m_fields[iCol][iRank]);
          break;
        case BLACK_QUEEN:
          boardPiece[iRank][iCol] = new alg2_pieceQueenBlack(m_board.m_fields[iCol][iRank]);
          break;
        case PIECE_KING:
          boardPiece[iRank][iCol] = new alg2_pieceKing(m_board.m_fields[iCol][iRank]);
          break;
        case BLACK_KING:
          boardPiece[iRank][iCol] = new alg2_pieceKingBlack(m_board.m_fields[iCol][iRank]);
          break;
        case FIELD_BORDER:
        default:
          // suppress compile warning on unhandled case
          // TODO: Optimize, field is only used in alg1_ board representation
          break;
      }
    }
  }
}

int alg2_board::generatemoves() {
  for (rank_t rank = 0; rank < 8; rank++) {
    for (col_t col = 0; col < 8; col++) {
    }
  }
  return getMovesFound();
}

/**
 * add moves to board position moves
 * @param m list of moves to add
 */
void alg2_board::addMoves(UNUSED const alg2_move_vec_t& m) {
}

alg2_move alg2_board::getSelectedMove(const int iMove) {
  return moves.getMove(iMove);
}

/**
 * does position have moves
 * @return true in case there is at least one move
 */
bool alg2_board::hasMoves() {
  return (moves.size() != 0);
}

int alg2_board::getMovesFound() {
  return moves.size();
}

t_piece alg2_board::getPieceAt(const rank_t rank, const col_t col) const {
  return boardPiece[rank][col]->getPiece();
}

t_piece alg2_board::getPieceAt(const alg2_boardpos& pos) const {
  return boardPiece[pos.getRank()][pos.getCol()]->getPiece();
}

/**
 * moves class, primary function is to contain a list of moves
 */
alg2_movelist::alg2_movelist() {

}

alg2_movelist::alg2_movelist(const alg2_movelist& orig) {
  // member element is board structure, gives compile warning
  // TODO: replace member by class structure
  moves = orig.moves;
}

alg2_movelist::~alg2_movelist() {

}

void alg2_movelist::addMove(const alg2_move& m) {
  moves.push_back(m);
}

void alg2_movelist::addMoves(const alg2_move_vec_t& m) {
  moves.insert(moves.end(), m.begin(), m.end());
}

int alg2_movelist::size() const {
  return moves.size();
}

/**
 * return move i
 * @param iSelectedMove
 * @return move
 */
alg2_move alg2_movelist::getMove(const int iSelectedMove) {
  return moves[iSelectedMove];
}

/**
 * move class, contains a single move
 */

alg2_move::alg2_move(const alg2_move& orig) {
  from = orig.from;
  to = orig.to;
}

alg2_move::~alg2_move() {

}

/**
 * board position class
 */

alg2_boardpos::alg2_boardpos(const alg2_boardpos& orig) {
  rank = orig.rank;
  col = orig.col;
}

/**
 * check if row-column indexes are not out of bounds
 * @return true if valid
 */
bool alg2_boardpos::isValid() const {
  if ((rank < 0) || (rank > 7) || (col < 0) || (col > 7)) return false;
  return true;
}

/**
 * piece base class
 */

alg2_piece::alg2_piece(UNUSED const alg2_piece& orig) {

}

alg2_piece::~alg2_piece() {

}

// TODO: remove need to have this function

void alg2_piece::invertColour() {

}

// TODO: remove need to have this function

int alg2_piece::generateMoves(UNUSED const alg2_board& board,
        UNUSED const rank_t rank, UNUSED const col_t col) {
  return 0;
}

/**
 * Add single move in single direction, either capture or empty field
 * @param board
 */
bool alg2_piece::tryMove(const alg2_board& board,
        UNUSED alg2_move_vec_t& movesparam,
        const alg2_boardpos& orig,
        const rank_t incRank, const col_t incCol) {
  // throw exception if out of bounds
  alg2_boardpos newPos;
  newPos.setPos(orig.getRank() + incRank, orig.getCol() + incCol);

  if (newPos.isValid()) {
    if (board.getPieceAt(newPos) == FIELD_EMPTY) {
      alg2_move nM(orig, newPos);
      //moves.push_back(nM);
      return true;
    }
  }
  return false;
}

/**
 * Add multiple moves in single direction, stop at own piece, border or capture
 */
void alg2_piece::tryDirection(UNUSED const alg2_board& board,
        UNUSED alg2_move_vec_t& movesparam,
        UNUSED const alg2_boardpos& orig,
        UNUSED const rank_t incRank, UNUSED const col_t incCol) {

}

/**
 * pawn class, used for white and black
 */
int alg2_piecePawn::generateMoves(UNUSED const alg2_board& board,
        const rank_t iRank, const col_t iCol) {
  alg2_boardpos orig;
  orig.setPos(iRank, iCol);

  return moves.size();
}

void alg2_piecePawn::invertColour() {
  pieceid = pieceid == PIECE_PAWN ? BLACK_PAWN : PIECE_PAWN;
}

/**
 * rook class, used for white and black
 */
int alg2_pieceRook::generateMoves(UNUSED const alg2_board& board,
        UNUSED const rank_t rank, UNUSED const col_t col) {
  return moves.size();
}

/**
 * bishop class, used for white and black
 */
int alg2_pieceBishop::generateMoves(UNUSED const alg2_board& board,
        UNUSED const rank_t rank, UNUSED const col_t col) {
  return moves.size();
}

/**
 * knight class, used for white and black
 */
int alg2_pieceKnight::generateMoves(UNUSED const alg2_board& board,
        UNUSED const rank_t rank, UNUSED const col_t col) {
  return moves.size();
}

/**
 * Queen class, used for white and black
 */
int alg2_pieceQueen::generateMoves(UNUSED const alg2_board& board,
        UNUSED const rank_t rank, UNUSED const col_t col) {
  return moves.size();
}

/**
 * King class, used for white and black
 */
int alg2_pieceKing::generateMoves(UNUSED const alg2_board& board,
        UNUSED const rank_t rank, UNUSED const col_t col) {
  return moves.size();
}
// eof
