/*
 * File:   alg2_search.cpp
 * Project: part of belofte - A Promising Chess Program
 * Author: yves
 *
 * Created on 6 mars 2019, 00:19
 */

#include "search.h"

#include "alg2_search.h"
#include "alg2_board.h"

/**
 * Copy constructor
 */
alg2_search::alg2_search(UNUSED const alg2_search& orig) {
}

/**
 * function called once for new game, here the board position is copied from the board into the engine
 * calling this function will cause ep, rock and repetition and 50 moves flags to be reset
 */
void alg2_search::initialboard(void) {
}

/**
 * Do perform search based on stored position, entry point for search
 * @return best score found
 */
t_score alg2_search::search() {
  fScore = -SCORE_WON;
  szSelectedMove[0] = '\0';
  uiBoard.copyboardfromui();
  if (searchFromPos(uiBoard)) {
    fScore = 0;
  }
  return fScore;
}

/**
 * Do perform search in position
 * @param pos the position to start search from
 * @return true in case search resulted in move
 * TODO: cost of non const parameter to be evaluated
 */
bool alg2_search::searchFromPos(alg2_board& pos) {
  if (pos.generatemoves() != 0) {
    alg2_move mSelectedMove = pos.getSelectedMove(0); // TODO: fix 0
    snprintf(szSelectedMove, MAX_COORDMOVE_LENGTH, "%c%c%c%c",
            mSelectedMove.from.getCol() + 'a',
            mSelectedMove.from.getRank() + '1',
            mSelectedMove.to.getCol() + 'a',
            mSelectedMove.to.getRank() + '1');
    return true;
  }
  return false;
}

/**
 * in check after best move found would be played
 * @param bEvalForWhite
 * @return true if in check
 */
bool alg2_search::isInCheck(UNUSED bool bEvalForWhite) {
  return false;
}

/**
 * fill move found as result of search
 * move returned should be in coordinate system
 * @param szMove
 */
void alg2_search::fillSelectedMove(char *szMove) {
  strncpy(szMove, szSelectedMove, sizeof (szSelectedMove));
}
// eof
