/***********************************************************************************
 * File: bel_debug.cpp
 * Project: part of belofte - A Promising Chess Program
 * Author: yves
 * Created on 23/01/2020
 ***********************************************************************************/

#include "belofte.h"

#include "usercmd.h"
#include "board.h"
#include "search.h"

// --------------------------------------------------------------------

extern uimode_t uimode;
extern game_t game;

// --------------------------------------------------------------------

static void debug_go(void);
static void debug_go1(void);
static void debug_fenhistory(void);
static void debug_hashhistory(void);

// --------------------------------------------------------------------

/**
 * activate debug, activate file logging or close all files
 * if logging to file, always truncate file
 * @param szArg [on|off|fenhistory|hashhistory|filename.ext]
 */
void cmd_debug(UNUSED const char *szCommand, const char *szArg) {
  if (strcmp(szArg, "fenhistory") == 0) {
    debug_fenhistory();
  } else if (strcmp(szArg, "hashhistory") == 0) {
    debug_hashhistory();
  } else if (strcmp(szArg, "off") == 0) {
    if (uimode.fpDebugOutput != NULL) {
      fclose(uimode.fpDebugOutput);
      uimode.fpDebugOutput = NULL;
      strcpy(uimode.szDebugFile, "");
    }
    uimode.debug = myFALSE;
  } else if (strcmp(szArg, "on") == 0) {
    uimode.debug = myTRUE;
  } else if (strcmp(szArg, "go") == 0) {
    debug_go();
  } else if (strcmp(szArg, "go1") == 0) {
    debug_go1();
  } else if (strlen(szArg) != 0) {
    cmd_debug("", "off"); // recursive call to make sure file is closed
    if (strlen(szArg) < MAX_DEBUGFILE_LENGTH - 1) {
      uimode.fpDebugOutput = fopen(szArg, "w+");
      if (uimode.fpDebugOutput != NULL) {
        strncpy(uimode.szDebugFile, szArg, sizeof (uimode.szDebugFile));
      }
    }
  } else {
    uimode.debug = myTRUE;
  }
}

static void debug_go(void) {
  int iSearchResult;
  char szMove[MAX_SANMOVE_LENGTH];
  char szResult[MAX_LINE_LENGTH];
  iSearchResult = search_root(szMove, sizeof (szMove), szResult, sizeof (szResult));
  cmdresp_snd_move(szMove, iSearchResult, szResult);
}

static void debug_go1(void) {
  cmd_fd("", "1");
  debug_go();
}

static void debug_fenhistory(void) {
  char szBuffer[MAX_DATANAME_LENGTH];
  t_boolean lDebug = uimode.debug;
  // temporary turn on debug mode
  uimode.debug = myTRUE;
  for (int i = game.firstply; i <= game.curply; i++) {
    snprintf(szBuffer, sizeof (szBuffer), "FEN ply %d: %s %d %d",
            i, game.m_boardhistory[i].szFEN, game.m_boardhistory[i].m_rule50,
            game.m_boardhistory[i].m_movenumber);
    cmdresp_snd_debug(szBuffer);
    if (i < game.curply) {
      snprintf(szBuffer, sizeof (szBuffer), "MOVE %s",
              game.m_boardhistory[i].szSANMove);
      cmdresp_snd_debug(szBuffer);
    }
  }
  uimode.debug = lDebug;
}

static void debug_hashhistory(void) {
  char szBuffer[MAX_DATANAME_LENGTH];
  t_boolean lDebug = uimode.debug;
  // temporary turn on debug mode
  uimode.debug = myTRUE;
  snprintf(szBuffer, sizeof (szBuffer), "Position ply %d: %s %d %d",
          game.firstply, game.m_boardhistory[game.firstply].szFEN,
          game.m_boardhistory[game.firstply].m_rule50,
          game.m_boardhistory[game.firstply].m_movenumber);
  cmdresp_snd_debug(szBuffer);
  for (int i = game.firstply; i <= game.curply; i++) {
    if (i < game.curply) {
      snprintf(szBuffer, sizeof (szBuffer), "HASH %llx - MOVE %s",
              game.m_boardhistory[i].m_hash, game.m_boardhistory[i].szSANMove);
      cmdresp_snd_debug(szBuffer);
    } else {
      snprintf(szBuffer, sizeof (szBuffer), "HASH %llx",
              game.m_boardhistory[i].m_hash);
      cmdresp_snd_debug(szBuffer);
    }
  }
  uimode.debug = lDebug;
}

/** TODO: check exact implementation of debug
 * current implementation is not checked against specifications
 */
void cmdresp_snd_debug(const char *szMessage) {
  char szBuffer[MAX_DATANAME_LENGTH];

  if (uimode.debug == myTRUE) {
    snprintf(szBuffer, sizeof (szBuffer), "# %s", szMessage);
    cmdresp_snd(szBuffer);
  }
  if (uimode.fpDebugOutput != NULL) {
    fprintf(uimode.fpDebugOutput, "%s\n", szMessage);
    fflush(uimode.fpDebugOutput);
  }
}

// eof
