/***********************************************************************************
 * File: board.h
 * Project: part of belofte - A Promising Chess Program
 * Author: yves
 ***********************************************************************************/

#if !defined(__BOARD_H__)
#define __BOARD_H__

#include "belofte.h"

// forward declaration
struct st_game;
typedef struct st_game game_t;

enum tPiece {
  FIELD_EMPTY = ' ',
  PIECE_BISHOP = 'B',
  PIECE_KING = 'K',
  PIECE_KNIGHT = 'N',
  PIECE_PAWN = 'P',
  PIECE_QUEEN = 'Q',
  PIECE_ROOK = 'R',
  BLACK_BISHOP = 'b',
  BLACK_KING = 'k',
  BLACK_KNIGHT = 'n',
  BLACK_PAWN = 'p',
  BLACK_QUEEN = 'q',
  BLACK_ROOK = 'r',
  FIELD_BORDER = '#'
};

typedef enum tPiece t_piece;

// hash values, index of pieces

enum tHPiece {
  HP_WP = 0, HP_WB, HP_WN, HP_WR, HP_WQ, HP_WK,
  HP_BP = 8, HP_BB, HP_BN, HP_BR, HP_BQ, HP_BK
};
typedef enum tHPiece t_hpiece;
#define HASH_PIECELIST_SIZE 14

// use entries in same array for special board information
#define HP_SPECIAL1 7
#define HP_SPECIAL1_L 6

enum tHSpecial1 {
  HP_TO_MOVE = 0, HP_WHITECASTLE_S, HP_WHITECASTLE_L,
  HP_BLACKCASTLE_S, HP_BLACKCASTLE_L, HP_EPCASE
};

// hash values, storage key type and size
typedef unsigned long long hashkey_t;

#define HASH_INIT 0x0000BE10F1E00000ULL

struct st_board {
  t_piece m_fields[8][8]; // board representation
  t_flag m_piecemoved[2][2]; // storage for rooks if moved (or both if king moved)
  t_ply m_ply; // ply number in the game
  t_ply m_rule50; // number since last pawn move or capture
  posid m_epfield; // where can an ep move take place, 0 if none?
  t_colour m_sidetomove : 2; // colour of player to move
  hashkey_t m_hash; // Zobrist derivate
};

typedef struct st_board board_t;

// board layout is left bottum index = 0, right bottom = 7, left top = 56, right top = 63
// board element is n_field[column][row]

/* structure containing membuffer with board and all moves and representations of each move played
 * used for undo and
 */
struct st_history {
  char szSANMove[MAX_SANMOVE_LENGTH];
  char szIntMove[MAX_COORDMOVE_LENGTH];
  char szFEN[MAX_FEN_LENGTH];
  t_ply m_movenumber; // move number in the game 1 - MAX_GAME_LENGTH/2
  t_ply m_rule50; // number since last pawn move or capture
  board_t m_board;
  hashkey_t m_hash; // Zobrist derivate
};

typedef struct st_history history_t;

/** structure to contain all of current game data
 */
struct st_game {
  int curply;
  int firstply;
  char szOpponentName[MAX_DATANAME_LENGTH];
  char szResult[MAX_DATANAME_LENGTH];
  char szSite[MAX_DATANAME_LENGTH];
  history_t m_boardhistory[MAX_GAME_LENGTH];
};

typedef struct st_game game_t;

// --------------------------------------------------------------------

enum tPromotion {
  PROMOTION_NONE = 0,
  PROMOTION_QUEEN = PIECE_QUEEN, PROMOTION_ROOK = PIECE_ROOK,
  PROMOTION_KNIGHT = PIECE_KNIGHT, PROMOTION_BISHOP = PIECE_BISHOP,
  PROMOTION_BLACKQUEEN = BLACK_QUEEN, PROMOTION_BLACKROOK = BLACK_ROOK,
  PROMOTION_BLACKKNIGHT = BLACK_KNIGHT, PROMOTION_BLACKBISHOP = BLACK_BISHOP,
  PROMOTION_PAWN = PIECE_PAWN, PROMOTION_BLACKPAWN = BLACK_PAWN
};

typedef enum tPromotion t_promotion;

// --------------------------------------------------------------------

// interface defines
void board_constructor(void);
void board_delayedconstructor(void);
int board_getmetadata(const char *szFlag);
hashkey_t board_calchash(void);
int board_drawinsufficientmat(void);
int board_positionrepeats(hashkey_t position);
bool board_draw50moves(const int nMovesAlert);
int board_sanapplymove(const char *szMove, char *szCoordMove);
int board_setupdefaultfen(void);
int board_setupfen(const char *szConstFen);
unsigned int board_setupepd(char *szEpd);
void board_sanmove(char *szDest, const char *szMove, const size_t iDestLen);
void board_print(void);
int board_applymove(const char *szCoordMove);
int board_unapplymove(void);
void board_fullfenstring(char *szDest, const char *szMove,
        t_ply *n50Moves, t_ply *nMoveNumber);
void board_fenstring(char *szDest, const char *szCoordMove);

// --------------------------------------------------------------------

#endif

// eof