/*
 * File:   search.h
 * Project: part of belofte - A Promising Chess Program
 * Author: yves
 *
 * Created on 6 mars 2019, 00:35
 */

#ifndef __SEARCH_H__
#define __SEARCH_H__

#include "belofte.h"

#include "board.h"

enum searchResult {
  SEARCH_ERROR = -1, SEARCH_FLAG = 0, SEARCH_OK = 1
};

#define SEARCHRESULT_STALEMATE "-1"
#define SEARCHRESULT_BLACKMATES "-2"
#define SEARCHRESULT_WHITEMATES "-3"
#define SEARCHRESULT_BLACKRESIGNS "-4"
#define SEARCHRESULT_WHITERESIGNS "-5"

struct bestmove_st {
  char szMove[MAX_COORDMOVE_LENGTH];
  t_score fMoveScore;
  hashkey_t m_hash; // Zobrist derivate
};

typedef struct bestmove_st bestmove_t;

struct st_search {
  t_boolean search_cancel;
  t_boolean engine_searching;
  t_depth bestmovedepth;
  t_score fSearchScore;
  t_score fInitialMaterialScore;
  bestmove_t bestmoves[MAX_SEARCH_DEPTH];
};

typedef struct st_search search_t;

// --------------------------------------------------------------------

void search_constructor(void);
void search_delayedconstructor(void);
void search_init(void);
int search_perft(void);
int search_root(char *szMove, const size_t iLenMove,
        char *szResult, const size_t iLenResult);

void search_inittimeformove(const int iPiecesOnBoard);
int search_adjusttimeformove(void);
void search_timeinformation(char *szInfo, const size_t nInfoLen);

void search_destructor(void);
int search_apply_move(const char *szMove);

#endif /* __SEARCH_H__ */

// eof