/*
 * File:   usercmd.h
 * Project: part of belofte - A Promising Chess Program
 * Author: yves
 *
 * Created on 7 juin 2017, 00:44
 */

#ifndef __USERCMD_H__
#define __USERCMD_H__

/** userinterface command handling
 */
#define CMD_NOPARAM        0
#define CMD_OPT            1
#define CMD_PARAM_STRING   2
#define CMD_PARAM_SWITCH   4
#define CMD_PARAM_NUMBER   8
#define CMD_PARAM_NOCHECK  16
#define CMD_NOTSUPPORTED   128

struct st_command {
  const char *cmd_name;
  void (*cmd_ptr)(const char *, const char *);
  const char *cmd_help;
  unsigned int cmd_interfacelevel : 4; // at what interface level this command can be used
  unsigned char cmd_paramdata; // give more detail about mandatory or optional parameters
};

typedef struct st_command command_t;

void usercmd_constructor(void);
void usercmd_delayedconstructor(void);

void cmd_quit(UNUSED const char *szCommand, UNUSED const char *szArg);
void cmd_new(UNUSED const char *szCommand, const char *szArg);
void cmd_go(UNUSED const char *szCommand, UNUSED const char *szArg);
void cmd_sd(UNUSED const char *szCommand, const char *szArg);
void cmd_fd(UNUSED const char *szCommand, const char *szArg);
void cmd_force(UNUSED const char *szCommand, UNUSED const char *szArg);
void cmd_about(UNUSED const char *szCommand, UNUSED const char *szArg);
void cmd_level(UNUSED const char *szCommand, const char *szArg);
void cmd_perft(UNUSED const char *szCommand, const char *szArg);
void cmd_post(UNUSED const char *szCommand, UNUSED const char *szArg);
void cmd_nopost(UNUSED const char *szCommand, UNUSED const char *szArg);
void cmd_debug(UNUSED const char *szCommand, const char *szArg);

int usercmd_readcommand(void);
int usercmd_execute(const char *szCommand, const char *szArg);

void cmdresp_snd(const char *szMessage);
void cmdresp_snd_info(const char *szMessage);
void cmdresp_snd_debug(const char *szMessage);
void cmdresp_snd_move(const char *szMove, const int iSearchResult, const char *szResult);
void cmdresp_snd_eval(const t_score fEval, const char *szThinking);
void cmdresp_snd_thinking(const int nPly, const char cPlyFlag, const t_score fEval,
        const char *szThinking);
void cmdresp_snd_prompt(void);

void game_movelist(FILE *fOutPut);

#endif /* __USERCMD_H__ */

// eof